@@ -9,23 +9,7 @@
  * Copyright (c) 2007-2015, Even Rouault <even.rouault at spatialys.com>
  * Copyright (c) 2015, Faza Mahamood
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -58,7 +42,8 @@
 #include "ogr_api.h"
 #include "ogr_srs_api.h"
 #include "ogr_spatialref.h"
-#include "ogrgeojsonreader.h"
+#include "ogrlibjsonutils.h"
+#include "ogrgeojsongeometry.h"
 #include "ogrgeojsonwriter.h"
 
 using std::vector;
@@ -105,6 +90,12 @@ struct GDALInfoOptions
     /*! force computation of the checksum for each band in the dataset */
     bool bComputeChecksum = false;
 
+    /*! allow or suppress printing of nodata value */
+    bool bShowNodata = true;
+
+    /*! allow or suppress printing of mask information */
+    bool bShowMask = true;
+
     /*! allow or suppress ground control points list printing. It may be useful
         for datasets with huge amount of GCPs, such as L1B AVHRR or HDF4 MODIS
         which contain thousands of them. */
@@ -129,7 +120,7 @@ struct GDALInfoOptions
     /*! report metadata for the specified domains. "all" can be used to report
         metadata in all domains.
         */
-    CPLStringList aosExtraMDDomains;
+    CPLStringList aosExtraMDDomains{};
 
     /*! WKT format used for SRS */
     std::string osWKTFormat = "WKT2";
@@ -261,6 +252,8 @@ GDALInfoAppOptionsGetParser(GDALInfoOptions *psOptions,
         .store_into(psOptions->bReportHistograms)
         .help(_("Report histogram information for all bands."));
 
+    argParser->add_usage_newline();
+
     argParser->add_inverted_logic_flag(
         "-nogcp", &psOptions->bShowGCPs,
         _("Suppress ground control points list printing."));
@@ -278,6 +271,15 @@ GDALInfoAppOptionsGetParser(GDALInfoOptions *psOptions,
     argParser->add_inverted_logic_flag("-nofl", &psOptions->bShowFileList,
                                        _("Suppress display of the file list."));
 
+    argParser->add_inverted_logic_flag(
+        "-nonodata", &psOptions->bShowNodata,
+        _("Suppress nodata printing (implies -nomask)."));
+
+    argParser->add_inverted_logic_flag("-nomask", &psOptions->bShowMask,
+                                       _("Suppress mask printing."));
+
+    argParser->add_usage_newline();
+
     argParser->add_argument("-checksum")
         .flag()
         .store_into(psOptions->bComputeChecksum)
@@ -921,10 +923,7 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
     {
         CPLErrorStateBackuper oErrorStateBackuper(CPLQuietErrorHandler);
 
-        json_object *poLinearRing = json_object_new_array();
         json_object *poCornerCoordinates = json_object_new_object();
-        json_object *poLongLatExtent = json_object_new_object();
-        json_object *poLongLatExtentType = json_object_new_string("Polygon");
         json_object *poLongLatExtentCoordinates = json_object_new_array();
 
         GDALInfoReportCorner(psOptions, hDataset, hTransform, "upperLeft", 0.0,
@@ -953,12 +952,26 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
 
         json_object_object_add(poJsonObject, "cornerCoordinates",
                                poCornerCoordinates);
-        json_object_object_add(poLongLatExtent, "type", poLongLatExtentType);
-        json_object_array_add(poLinearRing, poLongLatExtentCoordinates);
-        json_object_object_add(poLongLatExtent, "coordinates", poLinearRing);
-        json_object_object_add(poJsonObject,
-                               bTransformToWGS84 ? "wgs84Extent" : "extent",
-                               poLongLatExtent);
+
+        if (json_object_array_length(poLongLatExtentCoordinates) > 0)
+        {
+            json_object *poLinearRing = json_object_new_array();
+            json_object *poLongLatExtent = json_object_new_object();
+            json_object *poLongLatExtentType =
+                json_object_new_string("Polygon");
+            json_object_object_add(poLongLatExtent, "type",
+                                   poLongLatExtentType);
+            json_object_array_add(poLinearRing, poLongLatExtentCoordinates);
+            json_object_object_add(poLongLatExtent, "coordinates",
+                                   poLinearRing);
+            json_object_object_add(poJsonObject,
+                                   bTransformToWGS84 ? "wgs84Extent" : "extent",
+                                   poLongLatExtent);
+        }
+        else
+        {
+            json_object_put(poLongLatExtentCoordinates);
+        }
     }
     else if (GDALGetRasterXSize(hDataset))
     {
@@ -1069,6 +1082,9 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
                 case GDT_Int64:
                     stacDataType = "int64";
                     break;
+                case GDT_Float16:
+                    stacDataType = "float16";
+                    break;
                 case GDT_Float32:
                     stacDataType = "float32";
                     break;
@@ -1081,6 +1097,9 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
                 case GDT_CInt32:
                     stacDataType = "cint32";
                     break;
+                case GDT_CFloat16:
+                    stacDataType = "cfloat16";
+                    break;
                 case GDT_CFloat32:
                     stacDataType = "cfloat32";
                     break;
@@ -1111,25 +1130,21 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
             json_object_object_add(poStacEOBand, "name", poBandName);
         }
 
-        if (GDALGetDescription(hBand) != nullptr &&
-            strlen(GDALGetDescription(hBand)) > 0)
+        const char *pszBandDesc = GDALGetDescription(hBand);
+        if (pszBandDesc != nullptr && strlen(pszBandDesc) > 0)
         {
             if (bJson)
             {
-                json_object *poBandDescription =
-                    json_object_new_string(GDALGetDescription(hBand));
                 json_object_object_add(poBand, "description",
-                                       poBandDescription);
+                                       json_object_new_string(pszBandDesc));
 
-                json_object *poStacBandDescription =
-                    json_object_new_string(GDALGetDescription(hBand));
                 json_object_object_add(poStacEOBand, "description",
-                                       poStacBandDescription);
+                                       json_object_new_string(pszBandDesc));
             }
             else
             {
                 Concat(osStr, psOptions->bStdoutOutput, "  Description = %s\n",
-                       GDALGetDescription(hBand));
+                       pszBandDesc);
             }
         }
         else
@@ -1144,6 +1159,17 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
             }
         }
 
+        if (bJson)
+        {
+            const char *pszCommonName = GDALGetSTACCommonNameFromColorInterp(
+                GDALGetRasterColorInterpretation(hBand));
+            if (pszCommonName)
+            {
+                json_object_object_add(poStacEOBand, "common_name",
+                                       json_object_new_string(pszCommonName));
+            }
+        }
+
         {
             int bGotMin = FALSE;
             int bGotMax = FALSE;
@@ -1360,7 +1386,11 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
         }
 
         int bGotNodata = FALSE;
-        if (eDT == GDT_Int64)
+        if (!psOptions->bShowNodata)
+        {
+            // nothing to do
+        }
+        else if (eDT == GDT_Int64)
         {
             const auto nNoData =
                 GDALGetRasterNoDataValueAsInt64(hBand, &bGotNodata);
@@ -1466,8 +1496,12 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
                 if (bJson)
                 {
                     json_object *poNoDataValue =
-                        gdal_json_object_new_double_significant_digits(
-                            dfNoData, nSignificantDigits);
+                        (GDALDataTypeIsInteger(eDT) && dfNoData >= INT_MIN &&
+                         dfNoData <= INT_MAX &&
+                         static_cast<int>(dfNoData) == dfNoData)
+                            ? json_object_new_int(static_cast<int>(dfNoData))
+                            : gdal_json_object_new_double_significant_digits(
+                                  dfNoData, nSignificantDigits);
                     json_object *poStacNoDataValue = nullptr;
                     json_object_deep_copy(poNoDataValue, &poStacNoDataValue,
                                           nullptr);
@@ -1476,7 +1510,7 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
                     json_object_object_add(poBand, "noDataValue",
                                            poNoDataValue);
                 }
-                else if (CPLIsNan(dfNoData))
+                else if (std::isnan(dfNoData))
                 {
                     Concat(osStr, psOptions->bStdoutOutput,
                            "  NoData Value=nan\n");
@@ -1596,7 +1630,8 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
             Concat(osStr, psOptions->bStdoutOutput, "  Overviews: arbitrary\n");
         }
 
-        const int nMaskFlags = GDALGetMaskFlags(hBand);
+        const int nMaskFlags =
+            psOptions->bShowMask ? GDALGetMaskFlags(hBand) : GMF_ALL_VALID;
         if ((nMaskFlags & (GMF_NODATA | GMF_ALL_VALID)) == 0 ||
             nMaskFlags == (GMF_NODATA | GMF_PER_DATASET))
         {
@@ -1866,7 +1901,7 @@ char *GDALInfo(GDALDatasetH hDataset, const GDALInfoOptions *psOptions)
             {
                 json_object *poRAT =
                     static_cast<json_object *>(GDALRATSerializeJSON(hRAT));
-                json_object_object_add(poJsonObject, "rat", poRAT);
+                json_object_object_add(poBand, "rat", poRAT);
             }
             else
             {
@@ -2247,6 +2282,9 @@ static void GDALInfoReportMetadata(const GDALInfoOptions *psOptions,
         GDALInfoPrintMetadata(psOptions, hObject, "RPC", "RPC Metadata",
                               pszIndent, bJson, poMetadata, osStr);
     }
+
+    GDALInfoPrintMetadata(psOptions, hObject, "IMAGERY", "Imagery", pszIndent,
+                          bJson, poMetadata, osStr);
 }
 
 /************************************************************************/
@@ -2307,6 +2345,9 @@ GDALInfoOptionsNew(char **papszArgv,
         return nullptr;
     }
 
+    if (!psOptions->bShowNodata)
+        psOptions->bShowMask = false;
+
     return psOptions.release();
 }
 
