@@ -1791,10 +1791,7 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     if (poOpenInfo->eAccess == GA_Update)
     {
-        CPLError(
-            CE_Failure, CPLE_NotSupported,
-            "The SAR_CEOS driver does not support update access to existing"
-            " datasets.\n");
+        ReportUpdateNotSupportedByDriver("SAR_CEOS");
         return nullptr;
     }
 
@@ -1822,9 +1819,11 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Try the various filenames.                                      */
     /* -------------------------------------------------------------------- */
-    char *pszPath = CPLStrdup(CPLGetPath(poOpenInfo->pszFilename));
-    char *pszBasename = CPLStrdup(CPLGetBasename(poOpenInfo->pszFilename));
-    char *pszExtension = CPLStrdup(CPLGetExtension(poOpenInfo->pszFilename));
+    char *pszPath = CPLStrdup(CPLGetPathSafe(poOpenInfo->pszFilename).c_str());
+    char *pszBasename =
+        CPLStrdup(CPLGetBasenameSafe(poOpenInfo->pszFilename).c_str());
+    char *pszExtension =
+        CPLStrdup(CPLGetExtensionSafe(poOpenInfo->pszFilename).c_str());
 
     int nBand;
     if (strlen(pszBasename) > 4)
@@ -1841,7 +1840,7 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
         int e = 0;
         while (CeosExtension[e][iFile] != nullptr)
         {
-            char *pszFilename = nullptr;
+            std::string osFilename;
 
             /* build filename */
             if (EQUAL(CeosExtension[e][5], "base"))
@@ -1850,18 +1849,18 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
 
                 snprintf(szMadeBasename, sizeof(szMadeBasename),
                          CeosExtension[e][iFile], nBand);
-                pszFilename = CPLStrdup(
-                    CPLFormFilename(pszPath, szMadeBasename, pszExtension));
+                osFilename =
+                    CPLFormFilenameSafe(pszPath, szMadeBasename, pszExtension);
             }
             else if (EQUAL(CeosExtension[e][5], "ext"))
             {
-                pszFilename = CPLStrdup(CPLFormFilename(
-                    pszPath, pszBasename, CeosExtension[e][iFile]));
+                osFilename = CPLFormFilenameSafe(pszPath, pszBasename,
+                                                 CeosExtension[e][iFile]);
             }
             else if (EQUAL(CeosExtension[e][5], "whole"))
             {
-                pszFilename = CPLStrdup(
-                    CPLFormFilename(pszPath, CeosExtension[e][iFile], ""));
+                osFilename =
+                    CPLFormFilenameSafe(pszPath, CeosExtension[e][iFile], "");
             }
 
             // This is for SAR SLC as per the SAR Toolbox (from ASF).
@@ -1876,37 +1875,33 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
                     snprintf(szThisExtension, sizeof(szThisExtension), "%s",
                              CeosExtension[e][iFile]);
 
-                pszFilename = CPLStrdup(
-                    CPLFormFilename(pszPath, pszBasename, szThisExtension));
+                osFilename =
+                    CPLFormFilenameSafe(pszPath, pszBasename, szThisExtension);
             }
 
-            CPLAssert(pszFilename != nullptr);
-            if (pszFilename == nullptr)
-                return nullptr;
-
             /* try to open */
-            VSILFILE *process_fp = VSIFOpenL(pszFilename, "rb");
+            VSILFILE *process_fp = VSIFOpenL(osFilename.c_str(), "rb");
 
             /* try upper case */
             if (process_fp == nullptr)
             {
-                for (int i = static_cast<int>(strlen(pszFilename)) - 1;
-                     i >= 0 && pszFilename[i] != '/' && pszFilename[i] != '\\';
+                for (int i = static_cast<int>(osFilename.size()) - 1;
+                     i >= 0 && osFilename[i] != '/' && osFilename[i] != '\\';
                      i--)
                 {
-                    if (pszFilename[i] >= 'a' && pszFilename[i] <= 'z')
-                        pszFilename[i] = pszFilename[i] - 'a' + 'A';
+                    if (osFilename[i] >= 'a' && osFilename[i] <= 'z')
+                        osFilename[i] = osFilename[i] - 'a' + 'A';
                 }
 
-                process_fp = VSIFOpenL(pszFilename, "rb");
+                process_fp = VSIFOpenL(osFilename.c_str(), "rb");
             }
 
             if (process_fp != nullptr)
             {
-                CPLDebug("CEOS", "Opened %s.\n", pszFilename);
+                CPLDebug("CEOS", "Opened %s.\n", osFilename.c_str());
 
                 poDS->papszExtraFiles =
-                    CSLAddString(poDS->papszExtraFiles, pszFilename);
+                    CSLAddString(poDS->papszExtraFiles, osFilename.c_str());
 
                 CPL_IGNORE_RET_VAL(VSIFSeekL(process_fp, 0, SEEK_END));
                 if (ProcessData(process_fp, iFile, psVolume, -1,
@@ -1929,15 +1924,12 @@ GDALDataset *SAR_CEOSDataset::Open(GDALOpenInfo *poOpenInfo)
                     }
 
                     CPL_IGNORE_RET_VAL(VSIFCloseL(process_fp));
-                    CPLFree(pszFilename);
                     break; /* Exit the while loop, we have this data type*/
                 }
 
                 CPL_IGNORE_RET_VAL(VSIFCloseL(process_fp));
             }
 
-            CPLFree(pszFilename);
-
             e++;
         }
     }
