@@ -98,10 +98,19 @@ static int GetJXLDataType(TIFF *tif)
         return JXL_TYPE_FLOAT;
     }
 
-    TIFFErrorExtR(tif, module,
-                  "Unsupported combination of SampleFormat and BitsPerSample");
+    if (td->td_sampleformat == SAMPLEFORMAT_IEEEFP &&
+        td->td_bitspersample == 16)
+    {
+        return JXL_TYPE_FLOAT16;
+    }
+
+    TIFFErrorExtR(
+        tif, module,
+        "Unsupported combination of SampleFormat(=%d) and BitsPerSample(=%d)",
+        td->td_sampleformat, td->td_bitspersample);
     return -1;
 }
+
 static int GetJXLDataTypeSize(JxlDataType dtype)
 {
     switch (dtype)
@@ -112,6 +121,8 @@ static int GetJXLDataTypeSize(JxlDataType dtype)
             return 2;
         case JXL_TYPE_FLOAT:
             return 4;
+        case JXL_TYPE_FLOAT16:
+            return 2;
         default:
             return 0;
     }
@@ -402,8 +413,8 @@ static int JXLPreDecode(TIFF *tif, uint16_t s)
             if (buffer_size != channel_size)
             {
                 TIFFErrorExtR(tif, module,
-                              "JxlDecoderExtraChannelBufferSize returned %ld, "
-                              "expecting %u",
+                              "JxlDecoderExtraChannelBufferSize returned "
+                              "%" TIFF_SIZE_FORMAT ", expecting %u",
                               buffer_size, channel_size);
                 _TIFFfreeExt(tif, extra_channel_buffer);
                 return 0;
@@ -810,7 +821,10 @@ static int JXLPostEncode(TIFF *tif)
     basic_info.orientation = JXL_ORIENT_IDENTITY;
     if (td->td_sampleformat == SAMPLEFORMAT_IEEEFP)
     {
-        basic_info.exponent_bits_per_sample = 8;
+        if (td->td_bitspersample == 32)
+            basic_info.exponent_bits_per_sample = 8;
+        else
+            basic_info.exponent_bits_per_sample = 5;
     }
     else
     {
@@ -983,7 +997,7 @@ static int JXLPostEncode(TIFF *tif)
         int inStep = nBytesPerSample * td->td_samplesperpixel;
         uint8_t *cur_outbuffer = main_buffer;
         uint8_t *cur_inbuffer = sp->uncompressed_buffer;
-        for (; cur_outbuffer - main_buffer < main_size;
+        for (; (size_t)(cur_outbuffer - main_buffer) < main_size;
              cur_outbuffer += outChunkSize, cur_inbuffer += inStep)
         {
             memcpy(cur_outbuffer, cur_inbuffer, outChunkSize);
@@ -1036,6 +1050,21 @@ static int JXLPostEncode(TIFF *tif)
                     return 0;
                 }
             }
+            else if (!(sp->lossless))
+            {
+                // By default libjxl applies lossless encoding for extra channels
+                if (JXL_ENC_SUCCESS != JxlEncoderSetExtraChannelDistance(
+                                           opts, iExtraChannel, sp->distance))
+                {
+                    TIFFErrorExtR(
+                        tif, module,
+                        "JxlEncoderSetExtraChannelDistance(%d) failed",
+                        iChannel);
+                    JxlEncoderDestroy(enc);
+                    _TIFFfreeExt(tif, main_buffer);
+                    return 0;
+                }
+            }
 #endif
         }
     }
@@ -1147,14 +1176,14 @@ static void JXLCleanup(TIFF *tif)
 }
 
 static const TIFFField JXLFields[] = {
-    {TIFFTAG_JXL_LOSSYNESS, 0, 0, TIFF_ANY, 0, TIFF_SETGET_UINT32,
-     TIFF_SETGET_UNDEFINED, FIELD_PSEUDO, FALSE, FALSE, "Lossyness", NULL},
-    {TIFFTAG_JXL_EFFORT, 0, 0, TIFF_ANY, 0, TIFF_SETGET_UINT32,
-     TIFF_SETGET_UNDEFINED, FIELD_PSEUDO, FALSE, FALSE, "Effort", NULL},
-    {TIFFTAG_JXL_DISTANCE, 0, 0, TIFF_ANY, 0, TIFF_SETGET_FLOAT,
-     TIFF_SETGET_UNDEFINED, FIELD_PSEUDO, FALSE, FALSE, "Distance", NULL},
+    {TIFFTAG_JXL_LOSSYNESS, 0, 0, TIFF_ANY, 0, TIFF_SETGET_UINT32, FIELD_PSEUDO,
+     FALSE, FALSE, "Lossyness", NULL},
+    {TIFFTAG_JXL_EFFORT, 0, 0, TIFF_ANY, 0, TIFF_SETGET_UINT32, FIELD_PSEUDO,
+     FALSE, FALSE, "Effort", NULL},
+    {TIFFTAG_JXL_DISTANCE, 0, 0, TIFF_ANY, 0, TIFF_SETGET_FLOAT, FIELD_PSEUDO,
+     FALSE, FALSE, "Distance", NULL},
     {TIFFTAG_JXL_ALPHA_DISTANCE, 0, 0, TIFF_ANY, 0, TIFF_SETGET_FLOAT,
-     TIFF_SETGET_UNDEFINED, FIELD_PSEUDO, FALSE, FALSE, "AlphaDistance", NULL},
+     FIELD_PSEUDO, FALSE, FALSE, "AlphaDistance", NULL},
 };
 
 static int JXLVSetField(TIFF *tif, uint32_t tag, va_list ap)
@@ -1259,7 +1288,7 @@ int TIFFInitJXL(TIFF *tif, int scheme)
     JXLState *sp;
 
     (void)scheme;
-    assert(scheme == COMPRESSION_JXL);
+    assert(scheme == COMPRESSION_JXL || scheme == COMPRESSION_JXL_DNG_1_7);
 
     /*
      * Merge codec-specific tag information.
